@@ -48,7 +48,7 @@ static const cmdinfo_t	label_cmd =
 	{ "label", NULL, label_f, 0, 1, 1, "[label]",
 	  "write/print FS label", label_help };
 static const cmdinfo_t	version_cmd =
-	{ "version", NULL, version_f, 0, 1, 1, "[feature]",
+	{ "version", NULL, version_f, 0, -1, 1, "[feature | [vnum fnum]]",
 	  "set feature bit(s) in the sb version field", version_help };
 
 void
@@ -345,9 +345,9 @@ uuid_f(
 		}
 
 		if (!strcasecmp(argv[1], "generate")) {
-			uuid_generate(uu);
+			platform_uuid_generate(&uu);
 		} else if (!strcasecmp(argv[1], "nil")) {
-			uuid_clear(uu);
+			platform_uuid_clear(&uu);
 		} else if (!strcasecmp(argv[1], "rewrite")) {
 			uup = do_uuid(0, NULL);
 			if (!uup) {
@@ -355,10 +355,10 @@ uuid_f(
 				return 0;
 			}
 			memcpy(&uu, uup, sizeof(uuid_t));
-			uuid_unparse(uu, bp);
+			platform_uuid_unparse(&uu, bp);
 			dbprintf("old UUID = %s\n", bp);
 		} else {
-			if (uuid_parse(argv[1], uu)) {
+			if (platform_uuid_parse(argv[1], &uu)) {
 				dbprintf("invalid UUID\n");
 				return 0;
 			}
@@ -375,7 +375,7 @@ uuid_f(
 				break;
 			}
 
-		uuid_unparse(uu, bp);
+		platform_uuid_unparse(&uu, bp);
 		dbprintf("new UUID = %s\n", bp);
 		return 0;
 
@@ -408,7 +408,7 @@ uuid_f(
 				 "for FS with an external log\n");
 		}
 
-		uuid_unparse(uu, bp);
+		platform_uuid_unparse(&uu, bp);
 		dbprintf("UUID = %s\n", bp);
 	}
 
@@ -536,11 +536,12 @@ version_help(void)
 " 'version extflg'   - enable unwritten extents\n"
 " 'version attr1'    - enable v1 inline extended attributes\n"
 " 'version attr2'    - enable v2 inline extended attributes\n"
+" 'version log2'     - enable v2 log format\n"
 "\n"
 "The version function prints currently enabled features for a filesystem\n"
 "according to its the version field of the primary superblock.\n"
 "It can also be used to enable selected features, such as support for\n"
-"unwritten extents.  The upated version is written into to all AGs.\n"
+"unwritten extents.  The upated version is written into all AGs.\n"
 "\n"
 );
 }
@@ -549,14 +550,21 @@ static int
 do_version(xfs_agnumber_t agno, __uint16_t version, __uint32_t features)
 {
 	xfs_sb_t	tsb;
+	__int64_t	fields = 0;
 
 	if (!get_sb(agno, &tsb))
 		return 0;
 
+	if ((version & XFS_SB_VERSION_LOGV2BIT) &&
+	    !XFS_SB_VERSION_HASLOGV2(&tsb)) {
+		tsb.sb_logsunit = 1;
+		fields |= (1LL << XFS_SBS_LOGSUNIT);
+	}
+
 	tsb.sb_versionnum = version;
 	tsb.sb_features2 = features;
-	libxfs_xlate_sb(iocur_top->data, &tsb, -1,
-			(XFS_SB_VERSIONNUM | XFS_SBS_FEATURES2));
+	fields |= XFS_SB_VERSIONNUM | XFS_SB_FEATURES2;
+	libxfs_xlate_sb(iocur_top->data, &tsb, -1, fields);
 	write_cur();
 	return 1;
 }
@@ -641,6 +649,26 @@ version_f(
 						  XFS_SB_VERSION_EXTFLGBIT;
 				break;
 			}
+		} else if (!strcasecmp(argv[1], "log2")) {
+			switch (XFS_SB_VERSION_NUM(&mp->m_sb)) {
+			case XFS_SB_VERSION_1:
+				version = 0x0004 | XFS_SB_VERSION_LOGV2BIT;
+				break;
+			case XFS_SB_VERSION_2:
+				version = 0x0014 | XFS_SB_VERSION_LOGV2BIT;
+				break;
+			case XFS_SB_VERSION_3:
+				version = 0x0034 | XFS_SB_VERSION_LOGV2BIT;
+				break;
+			case XFS_SB_VERSION_4:
+				if (XFS_SB_VERSION_HASLOGV2(&mp->m_sb))
+					dbprintf("version 2 log format"
+						 " is already in use\n");
+				else
+					version = mp->m_sb.sb_versionnum |
+						  XFS_SB_VERSION_LOGV2BIT;
+				break;
+			}
 		} else if (!strcasecmp(argv[1], "attr1")) {
 			if (XFS_SB_VERSION_HASATTR2(&mp->m_sb)) {
 				if (!(mp->m_sb.sb_features2 &=
@@ -674,7 +702,24 @@ version_f(
 			mp->m_sb.sb_features2 = features;
 		}
 	}
+
+	if (argc == 3) {	/* VERSIONNUM + FEATURES2 */
+		char	*sp;
+
+		version = mp->m_sb.sb_versionnum;
+		features = mp->m_sb.sb_features2;
+		mp->m_sb.sb_versionnum = strtoul(argv[1], &sp, 0);
+		mp->m_sb.sb_features2 = strtoul(argv[2], &sp, 0);
+	}
+
 	dbprintf("versionnum [0x%x+0x%x] = %s\n", mp->m_sb.sb_versionnum,
 			mp->m_sb.sb_features2, version_string(&mp->m_sb));
+
+	if (argc == 3) {	/* now reset... */
+		mp->m_sb.sb_versionnum = version;
+		mp->m_sb.sb_features2 = features;
+		return 0;
+	}
+
 	return 0;
 }
