@@ -82,6 +82,7 @@ extern unsigned int	recurse_dir;
 
 extern void		*buffer;
 extern size_t		buffersize;
+extern int		align_direct(off64_t *, long long *);
 extern int		alloc_buffer(size_t, int, unsigned int);
 extern int		read_buffer(int, off64_t, long long, long long *,
 					int, int);
@@ -98,10 +99,12 @@ extern void		imap_init(void);
 extern void		inject_init(void);
 extern void		mmap_init(void);
 extern void		open_init(void);
+extern void		parent_init(void);
 extern void		pread_init(void);
 extern void		prealloc_init(void);
 extern void		pwrite_init(void);
 extern void		quit_init(void);
+extern void		shutdown_init(void);
 extern void		truncate_init(void);
 
 #ifdef HAVE_FADVISE
@@ -128,12 +131,6 @@ extern void		sendfile_init(void);
 #define sendfile_init()	do { } while (0)
 #endif
 
-#ifdef HAVE_SHUTDOWN
-extern void		shutdown_init(void);
-#else
-#define shutdown_init()	do { } while (0)
-#endif
-
 #ifdef HAVE_MADVISE
 extern void		madvise_init(void);
 #else
@@ -145,9 +142,3 @@ extern void		mincore_init(void);
 #else
 #define mincore_init()	do { } while (0)
 #endif
-
-#ifdef HAVE_PARENT
-extern void		parent_init(void);
-#else
-#define parent_init()	do { } while (0)
-#endif
