@@ -1,33 +1,19 @@
 /*
- * Copyright (c) 2000-2002 Silicon Graphics, Inc.  All Rights Reserved.
+ * Copyright (c) 2000-2002,2005 Silicon Graphics, Inc.
+ * All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of version 2 of the GNU General Public License as
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it would be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * This program is distributed in the hope that it would be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * Further, this software is distributed without any warranty that it is
- * free of the rightful claim of any third person regarding infringement
- * or the like.  Any license provided herein, whether implied or
- * otherwise, applies only to this software file.  Patent licenses, if
- * any, provided herein do not apply to combinations of this program with
- * other software, or any other product whatsoever.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
- *
- * Contact information: Silicon Graphics, Inc., 1600 Amphitheatre Pkwy,
- * Mountain View, CA  94043, or:
- *
- * http://www.sgi.com
- *
- * For further information regarding this notice, see:
- *
- * http://oss.sgi.com/projects/GenInfo/SGIGPLNoticeExplan/
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write the Free Software Foundation,
+ * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <libxfs.h>
@@ -40,6 +26,7 @@
 #include "dinode.h"
 #include "dir.h"
 #include "bmap.h"
+#include "prefetch.h"
 
 #if XFS_DIR_LEAF_MAPSIZE >= XFS_ATTR_LEAF_MAPSIZE
 #define XR_DA_LEAF_MAPSIZE	XFS_DIR_LEAF_MAPSIZE
@@ -120,7 +107,7 @@ process_shortform_dir(
 
 	sf = &dip->di_u.di_dirsf;
 
-	max_size = XFS_DFORK_DSIZE_ARCH(dip, mp, ARCH_CONVERT);
+	max_size = XFS_DFORK_DSIZE(dip, mp);
 	num_entries = INT_GET(sf->hdr.count, ARCH_CONVERT);
 	ino_dir_size = INT_GET(dip->di_core.di_size, ARCH_CONVERT);
 	*repair = 0;
@@ -145,7 +132,7 @@ process_shortform_dir(
 		sf_entry = next_sfe;
 		junkit = 0;
 		bad_sfnamelen = 0;
-		XFS_DIR_SF_GET_DIRINO_ARCH(&sf_entry->inumber, &lino, ARCH_CONVERT);
+		XFS_DIR_SF_GET_DIRINO(&sf_entry->inumber, &lino);
 
 		/*
 		 * if entry points to self, junk it since only '.' or '..'
@@ -451,7 +438,7 @@ process_shortform_dir(
 	/*
 	 * check parent (..) entry
 	 */
-	XFS_DIR_SF_GET_DIRINO_ARCH(&sf->hdr.parent, parent, ARCH_CONVERT);
+	XFS_DIR_SF_GET_DIRINO(&sf->hdr.parent, parent);
 
 	/*
 	 * if parent entry is bogus, null it out.  we'll fix it later .
@@ -465,8 +452,7 @@ process_shortform_dir(
 		if (!no_modify)  {
 			do_warn(_("clearing inode number\n"));
 
-			XFS_DIR_SF_PUT_DIRINO_ARCH(parent,
-					&sf->hdr.parent, ARCH_CONVERT);
+			XFS_DIR_SF_PUT_DIRINO(parent, &sf->hdr.parent);
 			*dino_dirty = 1;
 			*repair = 1;
 		} else  {
@@ -481,8 +467,7 @@ process_shortform_dir(
 	_("corrected root directory %llu .. entry, was %llu, now %llu\n"),
 				ino, *parent, ino);
 			*parent = ino;
-			XFS_DIR_SF_PUT_DIRINO_ARCH(parent,
-					&sf->hdr.parent, ARCH_CONVERT);
+			XFS_DIR_SF_PUT_DIRINO(parent, &sf->hdr.parent);
 			*dino_dirty = 1;
 			*repair = 1;
 		} else  {
@@ -501,8 +486,7 @@ process_shortform_dir(
 		if (!no_modify)  {
 			do_warn(_("clearing inode number\n"));
 
-			XFS_DIR_SF_PUT_DIRINO_ARCH(parent,
-					&sf->hdr.parent, ARCH_CONVERT);
+			XFS_DIR_SF_PUT_DIRINO(parent, &sf->hdr.parent);
 			*dino_dirty = 1;
 			*repair = 1;
 		} else  {
@@ -517,22 +501,7 @@ process_shortform_dir(
  * freespace map for directory leaf blocks (1 bit per byte)
  * 1 == used, 0 == free
  */
-static da_freemap_t dir_freemap[DA_BMAP_SIZE];
-
-#if 0
-unsigned char *
-alloc_da_freemap(xfs_mount_t *mp)
-{
-	unsigned char *freemap;
-
-	if ((freemap = malloc(mp->m_sb.sb_blocksize)) == NULL)
-		return(NULL);
-
-	bzero(freemap, mp->m_sb.sb_blocksize/NBBY);
-
-	return(freemap);
-}
-#endif
+size_t ts_dir_freemap_size = sizeof(da_freemap_t) * DA_BMAP_SIZE;
 
 void
 init_da_freemap(da_freemap_t *dir_freemap)
@@ -812,6 +781,9 @@ traverse_int_dablock(xfs_mount_t	*mp,
 	node = NULL;
 	da_cursor->active = 0;
 
+	if (do_prefetch && (whichfork == XFS_DATA_FORK))
+		prefetch_dir1(mp, bno, da_cursor);
+
 	do {
 		/*
 		 * read in each block along the way and set up cursor
@@ -1514,7 +1486,7 @@ junk_zerolen_dir_leaf_entries(
 
 			namest = XFS_DIR_LEAF_NAMESTRUCT(leaf, INT_GET(entry->nameidx, ARCH_CONVERT));
 			tmp_ino = NULLFSINO;
-			XFS_DIR_SF_PUT_DIRINO_ARCH(&tmp_ino, &namest->inumber, ARCH_CONVERT);
+			XFS_DIR_SF_PUT_DIRINO(&tmp_ino, &namest->inumber);
 			namest->name[0] = '/';
 
 			if (INT_GET(entry->nameidx, ARCH_CONVERT) < INT_GET(hdr->firstused, ARCH_CONVERT))
@@ -1583,8 +1555,8 @@ junk_zerolen_dir_leaf_entries(
 					map = &hdr->freemap[before];
 					INT_MOD(map->size, ARCH_CONVERT, sizeof(xfs_dir_leaf_entry_t));
 					INT_MOD(map->size, ARCH_CONVERT, INT_GET(hdr->freemap[after].size, ARCH_CONVERT));
-					INT_ZERO(hdr->freemap[after].base, ARCH_CONVERT);
-					INT_ZERO(hdr->freemap[after].size, ARCH_CONVERT);
+					hdr->freemap[after].base = 0;
+					hdr->freemap[after].size = 0;
 				} else if (before >= 0) {
 					map = &hdr->freemap[before];
 					INT_MOD(map->size, ARCH_CONVERT, sizeof(xfs_dir_leaf_entry_t));
@@ -1668,8 +1640,8 @@ junk_zerolen_dir_leaf_entries(
 				namest = XFS_DIR_LEAF_NAMESTRUCT(leaf,
 							INT_GET(entry->nameidx, ARCH_CONVERT));
 				tmp_ino = NULLFSINO;
-				XFS_DIR_SF_PUT_DIRINO_ARCH(&tmp_ino,
-							&namest->inumber, ARCH_CONVERT);
+				XFS_DIR_SF_PUT_DIRINO(&tmp_ino,
+							&namest->inumber);
 				namest->name[0] = '/';
 
 				bytes -= sizeof(xfs_dir_leaf_entry_t);
@@ -1682,7 +1654,7 @@ junk_zerolen_dir_leaf_entries(
 }
 #endif
 
-static char dirbuf[64 * 1024];
+size_t ts_dirbuf_size = 64*1024;
 
 /*
  * called by both node dir and leaf dir processing routines
@@ -1757,9 +1729,7 @@ process_leaf_dir_block(
 	char				fname[MAXNAMELEN + 1];
 	da_hole_map_t			holemap;
 	da_hole_map_t			bholemap;
-#if 0
-	unsigned char			*dir_freemap;
-#endif
+	unsigned char			*dir_freemap = ts_dir_freemap();
 
 #ifdef XR_DIR_TRACE
 	fprintf(stderr, "\tprocess_leaf_dir_block - ino %llu\n", ino);
@@ -1770,20 +1740,6 @@ process_leaf_dir_block(
 	 */
 	init_da_freemap(dir_freemap);
 
-#if 0
-	/*
-	 * XXX - alternatively, do this for parallel usage.
-	 * set up block freespace map.  head part of dir leaf block
-	 * including all entries are packed so we can use sizeof
-	 * and not worry about alignment.
-	 */
-
-	if ((dir_freemap = alloc_da_freemap(mp)) == NULL)  {
-		do_error(_("couldn't allocate directory block freemap\n"));
-		abort();
-	}
-#endif
-
 	*buf_dirty = 0;
 	first_used = mp->m_sb.sb_blocksize;
 	zero_len_entries = 0;
@@ -1872,8 +1828,8 @@ _("nameidx %d, entry #%d, bno %d, ino %llu > fs blocksize, marking entry bad\n")
 							INT_GET(entry->nameidx,
 								ARCH_CONVERT));
 					lino = NULLFSINO;
-					XFS_DIR_SF_PUT_DIRINO_ARCH(&lino,
-						&namest->inumber, ARCH_CONVERT);
+					XFS_DIR_SF_PUT_DIRINO(&lino,
+							&namest->inumber);
 					namest->name[0] = '/';
 				}
 			} else  {
@@ -1893,7 +1849,7 @@ _("nameidx %d, entry #%d, bno %d, ino %llu > fs blocksize, would delete entry\n"
 		 */
 		namest = XFS_DIR_LEAF_NAMESTRUCT(leaf,
 				INT_GET(entry->nameidx, ARCH_CONVERT));
-		XFS_DIR_SF_GET_DIRINO_ARCH(&namest->inumber, &lino, ARCH_CONVERT);
+		XFS_DIR_SF_GET_DIRINO(&namest->inumber, &lino);
 
 		/*
 		 * we may have to blow out an entry because of bad
@@ -1922,8 +1878,7 @@ _("nameidx %d, entry #%d, bno %d, ino %llu > fs blocksize, would delete entry\n"
 				_("\tclearing ino number in entry %d...\n"),
 					i);
 				lino = NULLFSINO;
-				XFS_DIR_SF_PUT_DIRINO_ARCH(&lino,
-					&namest->inumber, ARCH_CONVERT);
+				XFS_DIR_SF_PUT_DIRINO(&lino, &namest->inumber);
 				*buf_dirty = 1;
 			} else  {
 				do_warn(
@@ -1940,8 +1895,7 @@ _("entry #%d, bno %d in directory %llu references realtime bitmap inode %llu\n")
 					i);
 
 				lino = NULLFSINO;
-				XFS_DIR_SF_PUT_DIRINO_ARCH(&lino,
-					&namest->inumber, ARCH_CONVERT);
+				XFS_DIR_SF_PUT_DIRINO(&lino, &namest->inumber);
 				*buf_dirty = 1;
 			} else  {
 				do_warn(
@@ -1957,8 +1911,7 @@ _("entry #%d, bno %d in directory %llu references realtime summary inode %llu\n"
 				_("\tclearing ino number in entry %d...\n"), i);
 
 				lino = NULLFSINO;
-				XFS_DIR_SF_PUT_DIRINO_ARCH(&lino,
-					&namest->inumber, ARCH_CONVERT);
+				XFS_DIR_SF_PUT_DIRINO(&lino, &namest->inumber);
 				*buf_dirty = 1;
 			} else  {
 				do_warn(
@@ -1975,8 +1928,7 @@ _("entry #%d, bno %d in directory %llu references user quota inode %llu\n"),
 					i);
 
 				lino = NULLFSINO;
-				XFS_DIR_SF_PUT_DIRINO_ARCH(&lino,
-					&namest->inumber, ARCH_CONVERT);
+				XFS_DIR_SF_PUT_DIRINO(&lino, &namest->inumber);
 				*buf_dirty = 1;
 			} else  {
 				do_warn(
@@ -1993,8 +1945,7 @@ _("entry #%d, bno %d in directory %llu references group quota inode %llu\n"),
 					i);
 
 				lino = NULLFSINO;
-				XFS_DIR_SF_PUT_DIRINO_ARCH(&lino,
-					&namest->inumber, ARCH_CONVERT);
+				XFS_DIR_SF_PUT_DIRINO(&lino, &namest->inumber);
 				*buf_dirty = 1;
 			} else  {
 				do_warn(
@@ -2032,8 +1983,8 @@ _("entry #%d, bno %d in directory %llu references group quota inode %llu\n"),
 _("entry references free inode %llu in directory %llu, will clear entry\n"),
 						lino, ino);
 					lino = NULLFSINO;
-					XFS_DIR_SF_PUT_DIRINO_ARCH(&lino,
-						&namest->inumber, ARCH_CONVERT);
+					XFS_DIR_SF_PUT_DIRINO(&lino,
+							&namest->inumber);
 					*buf_dirty = 1;
 				} else  {
 					do_warn(
@@ -2050,8 +2001,7 @@ _("entry references free inode %llu in directory %llu, would clear entry\n"),
 			if (!no_modify)  {
 				do_warn(_("clearing inode number...\n"));
 				lino = NULLFSINO;
-				XFS_DIR_SF_PUT_DIRINO_ARCH(&lino,
-					&namest->inumber, ARCH_CONVERT);
+				XFS_DIR_SF_PUT_DIRINO(&lino, &namest->inumber);
 				*buf_dirty = 1;
 			} else  {
 				do_warn(_("would clear inode number...\n"));
@@ -2121,8 +2071,7 @@ _("entry references free inode %llu in directory %llu, would clear entry\n"),
 				namest = XFS_DIR_LEAF_NAMESTRUCT(leaf,
 						INT_GET(entry->nameidx,
 							ARCH_CONVERT));
-				XFS_DIR_SF_PUT_DIRINO_ARCH(&lino,
-					&namest->inumber, ARCH_CONVERT);
+				XFS_DIR_SF_PUT_DIRINO(&lino, &namest->inumber);
 				namest->name[0] = '/';
 			}
 		} else if (INT_GET(entry->nameidx, ARCH_CONVERT) +
@@ -2162,7 +2111,7 @@ _("dir entry slot %d in block %u conflicts with used space in dir inode %llu\n")
 		 */
 		bcopy(namest->name, fname, entry->namelen);
 		fname[entry->namelen] = '\0';
-		hashval = libxfs_da_hashname(fname, entry->namelen);
+		hashval = libxfs_da_hashname((uchar_t *) fname, entry->namelen);
 
 		/*
 		 * only complain about illegal names in phase 3 (when
@@ -2331,9 +2280,8 @@ _("name \"%s\" (block %u, slot %d) conflicts with used space in dir inode %llu\n
 						do_warn(
 		_("correcting .. entry in root inode %llu, was %llu\n"),
 							ino, *parent);
-						XFS_DIR_SF_PUT_DIRINO_ARCH(
-							&ino,
-						&namest->inumber, ARCH_CONVERT);
+						XFS_DIR_SF_PUT_DIRINO(
+							&ino, &namest->inumber);
 						*buf_dirty = 1;
 					} else  {
 						do_warn(
@@ -2375,9 +2323,8 @@ _("multiple .. entries in directory inode %llu, would clear second entry\n"),
 						do_warn(
 _(". in directory inode %llu has wrong value (%llu), fixing entry...\n"),
 							ino, lino);
-						XFS_DIR_SF_PUT_DIRINO_ARCH(&ino,
-							&namest->inumber,
-								ARCH_CONVERT);
+						XFS_DIR_SF_PUT_DIRINO(&ino,
+							&namest->inumber);
 						*buf_dirty = 1;
 					} else  {
 						do_warn(
@@ -2543,7 +2490,7 @@ _("- existing hole info for block %d, dir inode %llu (base, size) - \n"),
 			_("- compacting block %u in dir inode %llu\n"),
 					da_bno, ino);
 
-			new_leaf = (xfs_dir_leafblock_t *) &dirbuf[0];
+			new_leaf = (xfs_dir_leafblock_t *) ts_dirbuf();
 
 			/*
 			 * copy leaf block header
@@ -2635,10 +2582,10 @@ _("- existing hole info for block %d, dir inode %llu (base, size) - \n"),
 			ASSERT(INT_GET(new_leaf->hdr.freemap[0].base, ARCH_CONVERT) +
 				INT_GET(new_leaf->hdr.freemap[0].size, ARCH_CONVERT) == first_used);
 
-			INT_ZERO(new_leaf->hdr.freemap[1].base, ARCH_CONVERT);
-			INT_ZERO(new_leaf->hdr.freemap[1].size, ARCH_CONVERT);
-			INT_ZERO(new_leaf->hdr.freemap[2].base, ARCH_CONVERT);
-			INT_ZERO(new_leaf->hdr.freemap[2].size, ARCH_CONVERT);
+			new_leaf->hdr.freemap[1].base = 0;
+			new_leaf->hdr.freemap[1].size = 0;
+			new_leaf->hdr.freemap[2].base = 0;
+			new_leaf->hdr.freemap[2].size = 0;
 
 			/*
 			 * final step, copy block back
@@ -3025,8 +2972,8 @@ process_leaf_dir(
 			do_warn(_("clearing forw/back pointers for "
 				  "directory inode %llu\n"), ino);
 			buf_dirty = 1;
-			INT_ZERO(leaf->hdr.info.forw, ARCH_CONVERT);
-			INT_ZERO(leaf->hdr.info.back, ARCH_CONVERT);
+			leaf->hdr.info.forw = 0;
+			leaf->hdr.info.back = 0;
 		} else  {
 			do_warn(_("would clear forw/back pointers for "
 				  "directory inode %llu\n"), ino);
@@ -3073,7 +3020,7 @@ process_dir(
 	 * fix '.' and junk '..' if they're bogus.
 	 */
 	if (INT_GET(dip->di_core.di_size, ARCH_CONVERT) <=
-	    XFS_DFORK_DSIZE_ARCH(dip, mp, ARCH_CONVERT))  {
+	    XFS_DFORK_DSIZE(dip, mp))  {
 		dot = 1;
 		dotdot = 1;
 		if (process_shortform_dir(mp, ino, dip, ino_discovery,
