@@ -28,6 +28,8 @@
 #include "bmap.h"
 #include "versions.h"
 #include "dir2.h"
+#include "threads.h"
+#include "progress.h"
 
 
 /* ARGSUSED */
@@ -1118,6 +1120,18 @@ quota_sb_check(xfs_mount_t *mp)
 }
 
 
+void
+parallel_p4_process_aginodes(xfs_mount_t *mp, xfs_agnumber_t agno)
+{
+	do_log(_("        - agno = %d\n"), agno);
+	process_aginodes(mp, agno, 0, 1, 0);
+
+	/*
+	 * now recycle the per-AG duplicate extent records
+	 */
+	release_dup_extent_tree(agno);
+}
+
 void
 phase4(xfs_mount_t *mp)
 {
@@ -1141,6 +1155,8 @@ phase4(xfs_mount_t *mp)
 	do_log(_("Phase 4 - check for duplicate blocks...\n"));
 	do_log(_("        - setting up duplicate extent list...\n"));
 
+	set_progress_msg(PROG_FMT_DUP_EXTENT, (__uint64_t) glob_agcount);
+
 	irec = find_inode_rec(XFS_INO_TO_AGNO(mp, mp->m_sb.sb_rootino),
 				XFS_INO_TO_AGINO(mp, mp->m_sb.sb_rootino));
 
@@ -1238,7 +1254,9 @@ phase4(xfs_mount_t *mp)
 		 */
 		if (extent_start != 0)
 			add_dup_extent(i, extent_start, extent_len);
+		PROG_RPT_INC(prog_rpt_done[i], 1);
 	}
+	print_final_rpt();
 
 	/*
 	 * initialize realtime bitmap
@@ -1315,6 +1333,7 @@ phase4(xfs_mount_t *mp)
 	set_bmap_fs(mp);
 
 	do_log(_("        - check for inodes claiming duplicate blocks...\n"));
+	set_progress_msg(PROG_FMT_DUP_BLOCKS, (__uint64_t) mp->m_sb.sb_icount);
 	for (i = 0; i < mp->m_sb.sb_agcount; i++)  {
 		/*
 		 * ok, now process the inodes -- signal 2-pass check per inode.
@@ -1325,14 +1344,10 @@ phase4(xfs_mount_t *mp)
 		 * and attribute processing is turned OFF since we did that
 		 * already in phase 3.
 		 */
-		do_log(_("        - agno = %d\n"), i);
-		process_aginodes(mp, i, 0, 1, 0);
-
-		/*
-		 * now recycle the per-AG duplicate extent records
-		 */
-		release_dup_extent_tree(i);
+		queue_work(parallel_p4_process_aginodes, mp, i);
 	}
+	wait_for_workers();
+	print_final_rpt();
 
 	/*
 	 * free up memory used to track trealtime duplicate extents
