@@ -26,6 +26,8 @@
 #include "incore.h"
 #include "err_protos.h"
 #include "prefetch.h"
+#include "threads.h"
+#include "progress.h"
 
 #define	rounddown(x, y)	(((x)/(y))*(y))
 
@@ -63,6 +65,8 @@ char *o_opts[] = {
 	"pfdir",
 #define	PREFETCH_AIO_CNT	6
 	"pfaio",
+#define	THREAD_CNT		7
+	"thread",
 	NULL
 };
 
@@ -182,12 +186,13 @@ process_args(int argc, char **argv)
 	fs_has_extflgbit_allowed = 1;
 	pre_65_beta = 0;
 	fs_shared_allowed = 1;
+	report_interval = PROG_RPT_DEFAULT;
 
 	/*
 	 * XXX have to add suboption processing here
 	 * attributes, quotas, nlinks, aligned_inos, sb_fbits
 	 */
-	while ((c = getopt(argc, argv, "o:fl:r:LnDvVdP")) != EOF)  {
+	while ((c = getopt(argc, argv, "o:fl:r:LnDvVdPMt:")) != EOF)  {
 		switch (c) {
 		case 'D':
 			dumpcore = 1;
@@ -228,6 +233,9 @@ process_args(int argc, char **argv)
 				case PREFETCH_AIO_CNT:
 					libxfs_lio_aio_count = (int) strtol(val, 0, 0);
 					break;
+				case THREAD_CNT:
+					thread_count = (int) strtol(val, 0, 0);
+					break;
 				default:
 					unknown('o', val);
 					break;
@@ -255,7 +263,7 @@ process_args(int argc, char **argv)
 			dangerously = 1;
 			break;
 		case 'v':
-			verbose = 1;
+			verbose++;
 			break;
 		case 'V':
 			printf(_("%s version %s\n"), progname, VERSION);
@@ -263,6 +271,13 @@ process_args(int argc, char **argv)
 		case 'P':
 			do_prefetch ^= 1;
 			break;
+		case 'M':
+			do_parallel ^= 1;
+			break;
+		case 't':
+			report_interval = (int) strtol(optarg, 0, 0);
+			break;
+			
 		case '?':
 			usage();
 		}
@@ -458,6 +473,7 @@ main(int argc, char **argv)
 	xfs_sb_t	*sb;
 	xfs_buf_t	*sbp;
 	xfs_mount_t	xfs_m;
+	char		*msgbuf;
 
 	progname = basename(argv[0]);
 	setlocale(LC_ALL, "");
@@ -470,8 +486,12 @@ main(int argc, char **argv)
 	process_args(argc, argv);
 	xfs_init(&x);
 
+	timestamp(PHASE_START, 0, NULL);
+	timestamp(PHASE_END, 0, NULL);
+
 	/* do phase1 to make sure we have a superblock */
 	phase1(temp_mp);
+	timestamp(PHASE_END, 1, NULL);
 
 	if (no_modify && primary_sb_modified)  {
 		do_warn(_("Primary superblock would have been modified.\n"
@@ -505,6 +525,16 @@ main(int argc, char **argv)
 	max_symlink_blocks = howmany(MAXPATHLEN - 1, mp->m_sb.sb_blocksize);
 	inodes_per_cluster = XFS_INODE_CLUSTER_SIZE(mp) >> mp->m_sb.sb_inodelog;
 
+	if (do_parallel && report_interval) {
+		init_progress_rpt();
+		msgbuf = malloc(DURATION_BUF_SIZE);
+		if (msgbuf) {
+			do_log(_("        - reporting progress in intervals of %s\n"),
+			duration(report_interval, msgbuf));
+			free(msgbuf);
+		}
+	}
+
 	/*
 	 * calculate what mkfs would do to this filesystem
 	 */
@@ -522,18 +552,14 @@ main(int argc, char **argv)
 	}
 
 	/* make sure the per-ag freespace maps are ok so we can mount the fs */
-
 	phase2(mp);
+	timestamp(PHASE_END, 2, NULL);
 
-	if (verbose)
-		libxfs_report(stderr);
 	phase3(mp);
-	if (verbose)
-		libxfs_report(stderr);
+	timestamp(PHASE_END, 3, NULL);
 
 	phase4(mp);
-	if (verbose)
-		libxfs_report(stderr);
+	timestamp(PHASE_END, 4, NULL);
 
 	/* XXX: nathans - something in phase4 ain't playing by */
 	/* the buffer cache rules.. why doesn't IRIX hit this? */
@@ -541,15 +567,17 @@ main(int argc, char **argv)
 
 	if (no_modify)
 		printf(_("No modify flag set, skipping phase 5\n"));
-	else
+	else {
 		phase5(mp);
+	}
+	timestamp(PHASE_END, 5, NULL);
 
 	if (!bad_ino_btree)  {
 		phase6(mp);
-		if (verbose)
-			libxfs_report(stderr);
+		timestamp(PHASE_END, 6, NULL);
 
 		phase7(mp);
+		timestamp(PHASE_END, 7, NULL);
 	} else  {
 		do_warn(
 _("Inode allocation btrees are too corrupted, skipping phases 6 and 7\n"));
@@ -609,12 +637,14 @@ _("Warning:  project quota information would be cleared.\n"
 		}
 	}
 
-	if (verbose)
-		libxfs_report(stderr);
+	if (do_parallel && report_interval)
+		stop_progress_rpt();
 
 	if (no_modify)  {
 		do_log(
 	_("No modify flag set, skipping filesystem flush and exiting.\n"));
+		if (verbose)
+			summary_report();
 		if (fs_is_dirty)
 			return(1);
 
@@ -660,7 +690,8 @@ _("Note - stripe unit (%d) and width (%d) fields have been reset.\n"
 		libxfs_device_close(x.logdev);
 	libxfs_device_close(x.ddev);
 
+	if (verbose)
+		summary_report();
 	do_log(_("done\n"));
-
-	return(0);
+	return (0);
 }
